@@ -1,7 +1,8 @@
-__version__ = "0.4.1"
+__version__ = "0.6.4"
 __package__ = "oneai"
 
 from typing_extensions import Final
+import oneai.logger
 import oneai.skills as skills
 from oneai.classes import *
 from oneai.pipeline import Pipeline
@@ -24,5 +25,9 @@ Currently only enforced on `pipeline.run_batch`, other calls may be limited by t
 """
 PRINT_PROGRESS = True
 """
-Wether to print progress of batch processing in `Pipeline.run_batch` calls
+Does nothing. Set `logging.get('oneai').propagate = False` to disable logging
+"""
+DEBUG_RAW_RESPONSES = False
+"""
+Debug flag, return raw API responses instead of structured `Output` object. Only enable if you know what you're doing
 """
