@@ -1,14 +1,59 @@
-from base64 import b64encode
-from dataclasses import dataclass, field
+from datetime import datetime, timedelta
+import io
 import json
 import os
-from typing import Any, Callable, Dict, Iterable, List, Type, Union
+from base64 import b64encode
+import validators
+from dataclasses import dataclass, field
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    BinaryIO,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    TextIO,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 from warnings import warn
 
-import aiohttp
 
 from oneai.exceptions import InputError
 
+if TYPE_CHECKING:
+    from oneai.skills import OutputAttrs
+
+
+@dataclass
+class Utterance:
+    speaker: str
+    utterance: str
+
+    @classmethod
+    def from_dict(cls, u: Dict[str, str]) -> "Utterance":
+        return cls(u["speaker"], u["utterance"])
+
+    def __repr__(self) -> str:
+        return f"\n\t{self.speaker}: {self.utterance}"
+
+
+TextContent = TypeVar("TextContent", bound=Union[str, List["Utterance"]])
+PipelineInput = Union["Input[TextContent]", TextContent, TextIO, BinaryIO]
+
+# extension -> content_type, input_type
+CONTENT_TYPES: Dict[str, Tuple[str, str]] = {
+    ".json": ("application/json", "conversation"),
+    ".txt": ("text/plain", "article"),
+    ".srt": ("text/plain", "conversation"),
+    ".wav": ("audio/wav", "conversation"),
+    ".mp3": ("audio/mpeg", "conversation"),
+    ".html": ("text/plain", "article"),
+}
+
 
 @dataclass
 class Skill:
@@ -37,10 +82,6 @@ class Skill:
         The attribute name of the Skill's output in the Output object.
     `output_attr1: str`
         Only for Skills with 2 outputs (text / labels)
-    `run_custom: (Input, aiohttp.ClientSession) -> str | list[Label] | Output`
-        A custom function that will be called locally instead of passing the Skill to the API.
-        Can return a string with generated text (when `is_generator=True`), a list of `Label`s (when `is_generator=False`) or an `Output` object for more complex outputs.
-        Can use the `aiohttp.ClientSession` to make HTTP requests.
     """
 
     api_name: str = ""
@@ -50,9 +91,6 @@ class Skill:
     label_type: str = ""
     output_attr: str = ""
     output_attr1: str = field(default="", repr=False)
-    run_custom: Callable[
-        ["Input", aiohttp.ClientSession], Union[str, "Labels", "Output"]
-    ] = None
 
     def asdict(self) -> dict:
         return {
@@ -72,9 +110,6 @@ def skillclass(
     is_generator: bool = False,
     output_attr: str = "",
     output_attr1: str = "",
-    run_custom: Callable[
-        ["Input", aiohttp.ClientSession], Union[str, "Labels", "Output"]
-    ] = None,
 ):
     """
     A decorator for defining a Language Skill class. Decorate subclasses of `Skill` with this decorator to provide default values for instance attributes.
@@ -96,7 +131,6 @@ def skillclass(
                 is_generator=is_generator,
                 output_attr=output_attr,
                 output_attr1=output_attr1,
-                run_custom=run_custom,
             )
             self._skill_params = [
                 a
@@ -111,12 +145,14 @@ def skillclass(
     return wrap if cls is None else wrap(cls)
 
 
-class Input:
+class Input(Generic[TextContent]):
     """
     A base class for all input texts, allowing structured representations of inputs.
 
     ## Attributes
 
+    `text: TextContent`
+        Input text. Either `str` or `list[Utterance]` (conversation).
     `type: str`
         A type hint for the API, suggesting which models to use when processing the input.
     `content_type: str`
@@ -124,61 +160,62 @@ class Input:
     `encoding: str`
         The encoding of the input.
     `metadata: dict`
-        Optional metadata to be associated with the input in clustering collections. Not implemented by default. 
-
-    ## Methods
-
-    `get_text() -> str`
-        Returns the input as a string. Not implemented by default.
-    `parse(text) -> Input`
-        A class method. Parse a string into an instance of the Input class. Not implemented by default.
+        Optional metadata to be associated with the input in clustering collections. Not implemented by default.
     """
 
-    def __init__(self, type: str, content_type: str = None, encoding: str = None):
+    def __init__(
+        self,
+        text: TextContent,
+        *,
+        type: str,
+        content_type: str = None,
+        encoding: str = None,
+        metadata: Dict[str, any] = None,
+    ):
+        self.text: TextContent = text
         self.type = type
         self.content_type = content_type
         self.encoding = encoding
+        self.metadata = metadata
 
     @classmethod
-    def parse(cls, text: str) -> "Input":
-        """
-        A class method. Parse a string into an instance of the Input class. Not implemented by default.
-
-        ## Parameters
-
-        `text: str`
-            The text to parse.
-
-        ## Returns
-
-        The `Input` instance produced from `text`.
-        """
-        raise NotImplementedError()
-
-    @property
-    def raw(self) -> str:
-        """
-        Returns the input as a raw string. Not implemented by default.
-
-        ## Returns
-
-        `str` representation of this `Input` instance.
-        """
-        raise NotImplementedError()
-
-    @property
-    def metadata(self) -> Dict[str, Any]:
-        """
-        Returns the metadata of the input. Not implemented by default.
-
-        ## Returns
-
-        A `str`-key dict of input-associated metadata.
-        """
-        raise NotImplementedError()
+    def wrap(
+        cls, text: PipelineInput[TextContent], sync: bool = True
+    ) -> "Input[TextContent]":
+        if isinstance(text, cls):
+            return text
+        elif isinstance(text, str):
+            if validators.url(text):
+                return cls(text, type="article", content_type="text/uri-list")
+            else:
+                return cls(text, type="article", content_type="text/plain")
+        elif isinstance(text, list) and (
+            len(text) == 0 or isinstance(text[0], Utterance)
+        ):
+            return cls(text, type="conversation", content_type="application/json")
+        elif isinstance(text, io.IOBase):
+            name, mode = text.name, text.mode
+            _, ext = os.path.splitext(name)
+            if ext not in CONTENT_TYPES:
+                raise InputError(
+                    message=f"unsupported file extension {ext}",
+                    details="see supported files in docs",
+                )
+            content_type, input_type = CONTENT_TYPES[ext]
+            if "b" not in mode:
+                return cls(text.read(), type=input_type, content_type=content_type)
+            elif sync:
+                data = b64encode(text.read()).decode("ascii")
+                return cls(
+                    data, type=input_type, content_type=content_type, encoding="base64"
+                )
+            else:
+                return cls(text, type=input_type, content_type=content_type)
+        else:
+            raise ValueError(f"invalid content type {type(text)}")
 
 
-class Document(Input):
+class Document(Input[str]):
     """
     Represents any text that doesn't have a structured format
 
@@ -186,82 +223,39 @@ class Document(Input):
 
     `text: str`
         The text of the document.
-
-    ## Methods
-
-    `get_text() -> str`
-        Returns the text of the document.
-    `parse(text) -> Document`
-        A class method. Parse a string into a `Document` instance.
     """
 
-    type = "article"
-
     def __init__(self, text: str):
-        self.raw = text
-
-    @classmethod
-    def parse(cls, text: str) -> "Document":
-        """
-        A class method. Parse a string into a `Document` instance.
-
-        ## Parameters
-
-        `text: str`
-            The text to parse.
-
-        ## Returns
-
-        The `Document` instance produced from `text`.
-        """
-        return cls(text)
-
-    @property
-    def raw(self) -> str:
-        return self.text
+        super().__init__(text, type="article", content_type="text/plain")
 
 
-@dataclass
-class Utterance:
-    speaker: str
-    utterance: str
-
-    def __repr__(self) -> str:
-        return f"\n\t{self.speaker}: {self.utterance}"
-
-
-class Conversation(Input):
+class Conversation(Input[List[Utterance]]):
     """
     Represents conversations.
 
     ## Attributes
 
-    `utterances: List[Utterance]`
+    `text: List[Utterance]`
         A list of `Utterance` objects, each has `speaker` and `utterance` fields.
 
     ## Methods
 
-    `get_text() -> str`
-        Returns the conversation as a JSON string.
     `parse(text) -> Conversation`
         A class method. Parse a string with a structued conversation format or a conversation JSON string into a `Conversation` instance.
     """
 
-    type = "conversation"
-
     def __init__(self, utterances: List[Utterance] = []):
-        self.utterances = utterances
+        super().__init__(
+            utterances, type="conversation", content_type="application/json"
+        )
 
     @property
-    def raw(self) -> str:
-        """
-        Returns the conversation as a JSON string.
+    def utterances(self):
+        return self.text
 
-        ## Returns
-
-        `str` representation of this `Conversation` instance.
-        """
-        return json.dumps(self.utterances, default=lambda o: o.__dict__)
+    @utterances.setter
+    def utterances(self, value: List[Utterance]):
+        self.text = value
 
     def __getitem__(self, index: int) -> Utterance:
         return self.utterances[index]
@@ -290,7 +284,7 @@ class Conversation(Input):
         except json.JSONDecodeError:  # if not JSON, assume it's a structured conversation
             from oneai.parsing import parse_conversation
 
-            return parse_conversation(text)
+            return cls(parse_conversation(text))
 
     def __repr__(self) -> str:
         return f"oneai.Conversation{repr(self.utterances)}"
@@ -298,6 +292,8 @@ class Conversation(Input):
 
 class File(Input):
     """
+    Deprecated. Pass file-like objects directly to `Pipeline.run`
+
     Represents file inputs. Supported file extensions:
     * .txt (text files)
     * .wav, .mp3 (transcribe)
@@ -307,7 +303,7 @@ class File(Input):
 
     ## Attributes
 
-    `data: str`
+    `text: str`
         Encoded file data.
     `type: str`
         An input-type hint for the API, either `Conversation` or `Document`.
@@ -315,11 +311,6 @@ class File(Input):
         The content type of the file.
     `encoding: str`
         The encoding of the file.
-
-    ## Methods
-
-    `get_text() -> str`
-        Returns the encoded file data.
     """
 
     def __init__(self, file_path: str, type: Union[str, Type[Input]] = None):
@@ -333,6 +324,11 @@ class File(Input):
         `type: str` (optional)
             The input-type hint for the API, either `Conversation` or `Document`.
         """
+        warn(
+            "File class is deprecated- pass file-like objects directly to pipelines instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         if type is None or isinstance(type, str):
             super().__init__(type)
         elif issubclass(type, Input):
@@ -340,7 +336,7 @@ class File(Input):
         else:
             raise ValueError(f"Invalid input type {type}")
 
-        utf8, b64 = "utf8", "b64"
+        utf8, b64 = "utf8", "base64"
         _, ext = os.path.splitext(file_path)
         if ext == ".json":
             self.content_type = "application/json"
@@ -349,7 +345,7 @@ class File(Input):
             self.content_type = "text/plain"
             self.encoding = utf8
         elif ext == ".srt":
-            self.data = Conversation.parse(open(file_path).read()).raw
+            self.text = Conversation.parse(open(file_path).read()).text
             return
         elif ext in [".jpg", ".jpeg"]:
             self.content_type = "image/jpeg"
@@ -370,40 +366,18 @@ class File(Input):
             )
 
         if self.encoding == utf8:
-            self.data = open(file_path).read()
+            self.text = open(file_path, "r").read()
         else:
-            self.data = b64encode(file_path).decode("utf-8")
+            self.text = b64encode(open(file_path, "rb").read()).decode("utf-8")
 
-    @property
-    def raw(self) -> str:
-        """
-        Returns the encoded file data.
 
-        ## Returns
-
-        `str` representation of this `File` instance.
-        """
-
-        return self.data
-
-    @classmethod
-    def parse(cls, text: str) -> Input:
-        """
-        A class method. Parse a string into an `Input` instance.
-
-        ## Parameters
-
-        `text: str`
-            The text to parse.
-
-        ## Returns
-
-        The `Input` instance produced from `text`.
-        """
-        try:
-            return Conversation.parse(text)
-        except:
-            return text
+def timestamp_to_timedelta(timestamp: str) -> timedelta:
+    if not timestamp:
+        return None
+    dt = datetime.strptime(timestamp, "%H:%M:%S.%f")
+    return timedelta(
+        hours=dt.hour, minutes=dt.minute, seconds=dt.second, microseconds=dt.microsecond
+    )
 
 
 @dataclass
@@ -447,6 +421,10 @@ class Label:
         The spans in the input text that are relevant to the label. Only appears if the label was produced by a Skill that supports input spans.
     `span_text: str`
         The text of the label.
+    `timestamp: str`
+        For audio inputs, the timestamp of the start of the label.
+    `timestamp_end: str`
+        For audio inputs, the timestamp of the end of the label.
     `value: str`
         The value of the label.
     `data: Dict[str, Any]`
@@ -460,6 +438,8 @@ class Label:
     output_spans: List[int] = field(default_factory=list)
     input_spans: List[int] = field(default_factory=list)
     span_text: str = ""
+    timestamp: timedelta = None
+    timestamp_end: timedelta = None
     value: str = ""
     data: dict = field(default_factory=dict)
 
@@ -488,12 +468,18 @@ class Label:
             span_text=object.pop("span_text", ""),
             value=object.pop("value", ""),
             data=object.pop("data", {}),
+            timestamp=timestamp_to_timedelta(object.pop("timestamp", "")),
+            timestamp_end=timestamp_to_timedelta(object.pop("timestamp_end", "")),
         )
 
     def __repr__(self) -> str:
         return (
             "oneai.Label("
-            + ", ".join(f"{k}={repr(v)}" for k, v in self.__dict__.items() if v and not k.startswith("_"))
+            + ", ".join(
+                f"{k}={repr(v)}"
+                for k, v in self.__dict__.items()
+                if v and not k.startswith("_")
+            )
             + ")"
         )
 
@@ -537,15 +523,14 @@ class Labels(List[Label]):
         return [l.span_text for l in self]
 
 
-@dataclass
-class Output(Input):
+class Output(Input[TextContent], OutputAttrs if TYPE_CHECKING else object):
     """
     Represents the output of a pipeline. The structure of the output is dynamic, and corresponds to the Skills used and their order in the pipeline.
     Skill outputs can be accessed as attributes, either with the `api_name` of the corresponding Skill or the `output_attr` field.
 
     ## Attributes
 
-    `text: str`
+    `text: TextContent`
         The input text from which this `Output` instance was produced.
     `skills: List[Skill]`
         The Skills used to process `text` and produce this `Output` instance.
@@ -555,11 +540,22 @@ class Output(Input):
         * A nested `Output` instance, with a new `text`, e.g summary.
     """
 
-    text: Union[Input, str] = ""
-    skills: List[Skill] = field(
-        default_factory=list, repr=False
-    )  # not a dict since Skills are currently mutable & thus unhashable
-    data: List[Union[Labels, "Output"]] = field(default_factory=list, repr=False)
+    def __init__(
+        self,
+        text: TextContent,
+        skills: List[Skill] = [],
+        data: List[Union[Labels, "Output"]] = [],
+    ):
+        super().__init__(
+            text,
+            type="article" if isinstance(text, str) else "conversation",
+            content_type="text/plain" if isinstance(text, str) else "application/json",
+        )
+
+        self.skills = (
+            skills  # not a dict since Skills are currently mutable & thus unhashable
+        )
+        self.data = data
 
     def __getitem__(self, name: str) -> Union[Labels, "Output"]:
         return self.__getattr__(name)
@@ -583,36 +579,6 @@ class Output(Input):
             skill.output_attr or skill.api_name for skill in self.skills
         ]
 
-    @property
-    def raw(self) -> str:
-        return self.text if isinstance(self.text, str) else self.raw
-
-    def add(self, skill: Skill, data: Union["Output", Labels]):
-        """
-        Add data generated by a `Skill` to the output. Data can be a nested `Output` instance or a list of `Label`s.
-
-        ## Parameters
-
-        `skill: Skill`
-            The Skill which generated the data.
-        `data: Output | list[Label]`
-            The data to add to the output.
-        """
-        self.skills.append(skill)
-        self.data.append(data)
-
-    def merge(self, output: "Output") -> "Output":
-        """
-        Merge data from the given `Output` instance into this one, ignoring input text.
-
-        ## Parameters
-
-        `output: Output`
-            The `Output` instance to merge into this one.
-        """
-        self.skills += output.skills
-        self.data += output.data
-
     def __repr__(self) -> str:
         result = f"oneai.Output(text={repr(self.text)}"
         for i, skill in enumerate(self.skills):
