@@ -1,8 +1,8 @@
 from dataclasses import dataclass
+from typing import List
 from warnings import warn
-from oneai import scraping
 
-from oneai.classes import Skill, skillclass
+from oneai.classes import Document, Labels, Skill, Utterance, skillclass, Output
 
 
 @skillclass(
@@ -260,7 +260,6 @@ class Topics(Skill):
     api_name="extract-html",
     is_generator=True,
     output_attr="html_article",
-    run_custom=scraping.extract_article,
 )
 class HTMLExtractArticle(Skill):
     """
@@ -292,7 +291,11 @@ class HTMLExtractArticle(Skill):
         return super().__new__(cls, *args, **kwargs)
 
 
-@skillclass(api_name="html-extract-text", is_generator=True, output_attr="html_text", run_custom=scraping.extract_text)
+@skillclass(
+    api_name="html-extract-text",
+    is_generator=True,
+    output_attr="html_text",
+)
 class HTMLExtractText(Skill):
     """
     Extracts all text content of an HTML page (including text from control elements). Accepts URLs or HTML strings. Use HTMLExtractArticle to extract the main content only
@@ -325,7 +328,6 @@ class HTMLExtractText(Skill):
     api_name="extract-html",
     is_generator=True,
     output_attr="html_article",
-    run_custom=scraping.extract_article,
 )
 class HtmlToArticle(Skill):
     """
@@ -349,7 +351,11 @@ class HtmlToArticle(Skill):
     """
 
 
-@skillclass(api_name="html-extract-text", is_generator=True, output_attr="html_text", run_custom=scraping.extract_text)
+@skillclass(
+    api_name="html-extract-text",
+    is_generator=True,
+    output_attr="html_text",
+)
 class HtmlAllText(Skill):
     """
     Extracts all text content of an HTML page (including text from control elements). Accepts URLs or HTML strings. Use HTMLExtractArticle to extract the main content only
@@ -561,6 +567,7 @@ class SplitBySentence(Skill):
     label_type="dialogue-segment",
     output_attr="segments",
 )
+@dataclass
 class SplitByTopic(Skill):
     """
     Splits input by discussed topics
@@ -579,6 +586,9 @@ class SplitByTopic(Skill):
     >>> output.segments
     """
 
+    std_ratio: float = 0
+    use_discourse_parser: bool = False
+
 
 @skillclass(
     api_name="sales-insights", label_type="sales-insights", output_attr="sales_insights"
@@ -600,3 +610,118 @@ class SalesInsights(Skill):
     >>> output = pipeline.run('...')
     >>> output.sales_insights
     """
+
+
+@skillclass(
+    api_name="detect-language", label_type="detect-language", output_attr="language"
+)
+class DetectLanguage(Skill):
+    """
+    Detects language of input
+
+    ## Output Attributes
+
+    `language: list[Label]`
+        A list of `Label` objects, containing the detected language
+
+    ## Example
+
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.DetectLanguage()
+    ... ])
+    >>> output = pipeline.run('...')
+    >>> output.language[0]
+    """
+
+
+@skillclass(api_name="headline", label_type="headline")
+class Headline(Skill):
+    """
+    Generates a headline based on input
+
+    ## Output Attributes
+
+    `headline: list[Label]`
+        A list of `Label` objects, containing the generated headline
+
+    ## Example
+
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.Headline()
+    ... ])
+    >>> output = pipeline.run('...')
+    >>> output.headline[0]
+    """
+
+
+@skillclass(api_name="subheading", label_type="subheading", output_attr="subheading")
+class Subheading(Skill):
+    """
+    Generates a subheading based on input
+
+    ## Output Attributes
+
+    `subheading: list[Label]`
+        A list of `Label` objects, containing the generated subheading
+
+    ## Example
+
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.Subheading()
+    ... ])
+    >>> output = pipeline.run('...')
+    >>> output.subheading[0]
+    """
+
+
+@skillclass(api_name="transcribe", is_generator=True, output_attr="transcription", output_attr1="words")
+@dataclass
+class Transcribe(Skill):
+    """
+    Transcribes audio files
+
+    ## Output
+
+    Speech-to-text transcription of the audio data
+
+    ## Example
+
+    >>> input = oneai.File('./my_audio_file.wav')
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.Transcribe()
+    ... ])
+    >>> output = pipeline.run(input)
+    >>> str(output.transcription.text)[:20] + ' ...'
+    oneai.Conversation([ ...
+    """
+
+    speaker_detection: bool = True
+    timestamp_per_word: bool = False
+
+
+class OutputAttrs:
+    summary: "Output[str]" = None
+    proofread: "Output[List[Utterance]]" = None
+    html_article: "Output[str]" = None
+    html_text: "Output[str]" = None
+    transcription: "Output[List[Utterance]]" = None
+    anonymized: "Output" = None
+    words: Labels = None
+    emotions: Labels = None
+    names: Labels = None
+    numbers: Labels = None
+    keywords: Labels = None
+    highlights: Labels = None
+    sentiments: Labels = None
+    topics: Labels = None
+    sales_insights: Labels = None
+    action_items: Labels = None
+    pricing: Labels = None
+    replacements: Labels = None
+    origins: Labels = None
+    segments: Labels = None
+    sentences: Labels = None
+    anonymizations: Labels = None
+    language: Labels = None
+    headline: Labels = None
+    subheading: Labels = None
